@@ -45,37 +45,34 @@
  **/
 
 
-typedef struct _GMimeHeader GMimeHeader;
-
-struct _GMimeHeaderList {
-	GHashTable *writers;
-	GHashTable *hash;
-	List iters;
-	List list;
-	char *raw;
-};
-
 /**
  * GMimeHeader:
+ * @next: pointer to the next header
+ * @prev: pointer to the previous header
+ * @offset: file/stream offset
+ * @name: header name
+ * @value: header value
  *
  * A message/rfc822 header.
  **/
+
 struct _GMimeHeader {
-	struct _GMimeHeader *next;
-	struct _GMimeHeader *prev;
+	GMimeHeader *next;
+	GMimeHeader *prev;
 	gint64 offset;
 	char *name;
 	char *value;
 };
 
-struct _GMimeHeaderIter {
-	struct _GMimeHeaderIter *next;
-	struct _GMimeHeaderIter *prev;
-	GMimeHeaderList *headers;
-	GMimeHeader *cursor;
+struct _GMimeHeaderList {
+	GHashTable *writers;
+	GHashTable *hash;
+	guint32 version;
+	List list;
+	char *raw;
 };
 
-static void g_mime_header_list_invalidate_iters (GMimeHeaderList *headers, GMimeHeader *header);
+
 static GMimeHeader *g_mime_header_new (const char *name, const char *value, gint64 offset);
 static void g_mime_header_free (GMimeHeader *header);
 
@@ -121,13 +118,38 @@ g_mime_header_free (GMimeHeader *header)
 }
 
 
+/**
+ * g_mime_header_iter_new:
+ *
+ * Dynamically allocates a #GMimeHeaderIter on the heap. This is
+ * intended for use in language bindings but may also be useful in
+ * applications as well. You must free this iter with
+ * g_mime_header_iter_free().
+ *
+ * Returns: a newly-allocated #GMimeHeaderIter.
+ **/
+GMimeHeaderIter *
+g_mime_header_iter_new (void)
+{
+	GMimeHeaderIter *iter;
+	
+	iter = g_new (GMimeHeaderIter, 1);
+	iter->hdrlist = NULL;
+	iter->cursor = NULL;
+	iter->version = 0;
+	
+	return iter;
+}
+
+
 /**
  * g_mime_header_iter_copy:
  * @iter: a #GMimeHeaderIter
  *
- * Copies a header iterator.
+ * Creates a dynamically allocated header iterator as a copy of
+ * @iter. You must free this iter with g_mime_header_iter_free().
  *
- * Returns: a new #GMimeHeaderIter which matches @iter's state.
+ * Returns: a newly-allocated copy of @iter.
  **/
 GMimeHeaderIter *
 g_mime_header_iter_copy (GMimeHeaderIter *iter)
@@ -136,31 +158,42 @@ g_mime_header_iter_copy (GMimeHeaderIter *iter)
 	
 	g_return_val_if_fail (iter != NULL, NULL);
 	
-	copy = g_new (GMimeHeaderIter, 1);
-	copy->headers = iter->headers;
-	copy->cursor = iter->cursor;
-	
-	if (iter->headers)
-		list_append (&iter->headers->iters, (ListNode *) copy);
+	copy = g_mime_header_iter_new ();
+	memcpy (copy, iter, sizeof (GMimeHeaderIter));
 	
 	return copy;
 }
 
 
+/**
+ * g_mime_header_iter_copy_to:
+ * @src: a #GMimeHeaderIter
+ * @dest: a #GMimeHeaderIter
+ *
+ * Copies @src to @dest.
+ **/
+void
+g_mime_header_iter_copy_to (GMimeHeaderIter *src, GMimeHeaderIter *dest)
+{
+	g_return_if_fail (dest != NULL);
+	g_return_if_fail (src != NULL);
+	
+	memcpy (dest, src, sizeof (GMimeHeaderIter));
+}
+
+
 /**
  * g_mime_header_iter_free:
  * @iter: a #GMimeHeaderIter
  *
- * Frees a #GMimeHeaderIter.
+ * Frees a dynamically-allocated #GMimeHeaderIter as created by
+ * g_mime_header_iter_new() or g_mime_header_iter_copy().
  **/
 void
 g_mime_header_iter_free (GMimeHeaderIter *iter)
 {
 	g_return_if_fail (iter != NULL);
 	
-	if (iter->headers)
-		list_unlink ((ListNode *) iter);
-	
 	g_free (iter);
 }
 
@@ -181,7 +214,9 @@ g_mime_header_iter_equal (GMimeHeaderIter *iter1, GMimeHeaderIter *iter2)
 	g_return_val_if_fail (iter1 != NULL, FALSE);
 	g_return_val_if_fail (iter2 != NULL, FALSE);
 	
-	return iter1->cursor == iter2->cursor;
+	return iter1->hdrlist == iter2->hdrlist &&
+		iter1->version == iter2->version &&
+		iter1->cursor == iter2->cursor;
 }
 
 
@@ -190,8 +225,8 @@ g_mime_header_iter_equal (GMimeHeaderIter *iter1, GMimeHeaderIter *iter2)
  * @iter: a #GMimeHeaderIter
  *
  * Checks if a #GMimeHeaderIter is valid. An iterator may become
- * invalid if the #GMimeHeaderList that the iterator refers to changes
- * or is destroyed.
+ * invalid if the #GMimeHeaderList that the iterator refers to
+ * changes.
  *
  * Returns: %TRUE if @iter is still valid or %FALSE otherwise.
  **/
@@ -200,7 +235,10 @@ g_mime_header_iter_is_valid (GMimeHeaderIter *iter)
 {
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
-	return iter->cursor && iter->cursor->next;
+	if (!iter->hdrlist || !iter->cursor || !iter->cursor->next)
+		return FALSE;
+	
+	return iter->version == iter->hdrlist->version;
 }
 
 
@@ -220,13 +258,14 @@ g_mime_header_iter_first (GMimeHeaderIter *iter)
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
 	/* make sure we can actually do as requested */
-	if (!iter->headers)
+	if (!iter->hdrlist)
 		return FALSE;
 	
-	first = (GMimeHeader *) iter->headers->list.head;
+	first = (GMimeHeader *) iter->hdrlist->list.head;
 	if (!first->next)
 		return FALSE;
 	
+	iter->version = iter->hdrlist->version;
 	iter->cursor = first;
 	
 	return TRUE;
@@ -249,13 +288,14 @@ g_mime_header_iter_last (GMimeHeaderIter *iter)
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
 	/* make sure we can actually do as requested */
-	if (!iter->headers)
+	if (!iter->hdrlist)
 		return FALSE;
 	
-	last = (GMimeHeader *) iter->headers->list.tailpred;
+	last = (GMimeHeader *) iter->hdrlist->list.tailpred;
 	if (!last->next)
 		return FALSE;
 	
+	iter->version = iter->hdrlist->version;
 	iter->cursor = last;
 	
 	return TRUE;
@@ -278,7 +318,7 @@ g_mime_header_iter_next (GMimeHeaderIter *iter)
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
 	/* make sure current cursor is valid */
-	if (!iter->cursor || !iter->cursor->next)
+	if (!g_mime_header_iter_is_valid (iter))
 		return FALSE;
 	
 	/* make sure next item is valid */
@@ -308,12 +348,12 @@ g_mime_header_iter_prev (GMimeHeaderIter *iter)
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
 	/* make sure current cursor is valid */
-	if (!iter->cursor || !iter->cursor->prev)
+	if (!g_mime_header_iter_is_valid (iter))
 		return FALSE;
 	
 	/* make sure prev item is valid */
 	prev = iter->cursor->prev;
-	if (!prev->prev)
+	if (!prev || !prev->prev)
 		return FALSE;
 	
 	iter->cursor = prev;
@@ -335,7 +375,7 @@ g_mime_header_iter_get_offset (GMimeHeaderIter *iter)
 {
 	g_return_val_if_fail (iter != NULL, -1);
 	
-	if (!iter->cursor || !iter->cursor->next)
+	if (!g_mime_header_iter_is_valid (iter))
 		return -1;
 	
 	return iter->cursor->offset;
@@ -355,7 +395,7 @@ g_mime_header_iter_get_name (GMimeHeaderIter *iter)
 {
 	g_return_val_if_fail (iter != NULL, NULL);
 	
-	if (!iter->cursor || !iter->cursor->next)
+	if (!g_mime_header_iter_is_valid (iter))
 		return NULL;
 	
 	return iter->cursor->name;
@@ -377,13 +417,13 @@ g_mime_header_iter_set_value (GMimeHeaderIter *iter, const char *value)
 {
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
-	if (!iter->cursor || !iter->cursor->next)
+	if (!g_mime_header_iter_is_valid (iter))
 		return FALSE;
 	
 	g_free (iter->cursor->value);
 	iter->cursor->value = g_strdup (value);
 	
-	g_mime_header_list_set_raw (iter->headers, NULL);
+	g_mime_header_list_set_raw (iter->hdrlist, NULL);
 	
 	return TRUE;
 }
@@ -402,7 +442,7 @@ g_mime_header_iter_get_value (GMimeHeaderIter *iter)
 {
 	g_return_val_if_fail (iter != NULL, NULL);
 	
-	if (!iter->cursor || !iter->cursor->next)
+	if (!g_mime_header_iter_is_valid (iter))
 		return NULL;
 	
 	return iter->cursor->value;
@@ -415,37 +455,40 @@ g_mime_header_iter_get_value (GMimeHeaderIter *iter)
  *
  * Removes the current header and advances to the next header.
  *
- * Returns: %TRUE on success or %FALSE otherwise
+ * Note: If you remove the last header in the list, then @iter will
+ * become invalid regardless of whether or not other headers remain.
+ *
+ * Returns: %TRUE on success or %FALSE otherwise.
  **/
 gboolean
 g_mime_header_iter_remove (GMimeHeaderIter *iter)
 {
 	GMimeHeader *cursor, *header, *next;
-	GMimeHeaderList *headers;
+	GMimeHeaderList *hdrlist;
 	
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
-	if (!iter->cursor || !iter->cursor->next)
+	if (!g_mime_header_iter_is_valid (iter))
 		return FALSE;
 	
 	/* save iter state */
-	headers = iter->headers;
+	hdrlist = iter->hdrlist;
 	cursor = iter->cursor;
 	next = cursor->next;
 	
-	if (!(header = g_hash_table_lookup (headers->hash, cursor->name)))
+	if (!(header = g_hash_table_lookup (hdrlist->hash, cursor->name)))
 		return FALSE;
 	
 	if (cursor == header) {
 		/* update the header lookup table */
 		GMimeHeader *node = next;
 		
-		g_hash_table_remove (headers->hash, cursor->name);
+		g_hash_table_remove (hdrlist->hash, cursor->name);
 		
 		while (node->next) {
 			if (!g_ascii_strcasecmp (node->name, cursor->name)) {
 				/* enter this node into the lookup table */
-				g_hash_table_insert (headers->hash, node->name, node);
+				g_hash_table_insert (hdrlist->hash, node->name, node);
 				break;
 			}
 			
@@ -454,53 +497,18 @@ g_mime_header_iter_remove (GMimeHeaderIter *iter)
 	}
 	
 	/* remove/free the header */
-	g_mime_header_list_invalidate_iters (iter->headers, cursor);
 	list_unlink ((ListNode *) cursor);
 	g_mime_header_free (cursor);
+	hdrlist->version++;
 	
-	/* restore iter state */
-	iter->headers = headers;
+	/* update iter state */
+	iter->version = hdrlist->version;
 	iter->cursor = next;
 	
 	return TRUE;
 }
 
 
-/**
- * g_mime_header_list_invalidate_iters:
- * @headers: a #GMimeHeaderList
- * @header: a #GMimeHeader
- *
- * Invalidate all outstanding iterators that are currently referencing
- * @header. If @header is %NULL, then invalidate all iterators.
- **/
-static void
-g_mime_header_list_invalidate_iters (GMimeHeaderList *headers, GMimeHeader *header)
-{
-	GMimeHeaderIter *iter, *next;
-	
-	/* invalidate all our outstanding iterators matching @header */
-	iter = (GMimeHeaderIter *) headers->iters.head;
-	while (iter->next) {
-		next = iter->next;
-		
-		if (!header || iter->cursor == header) {
-			/* invalidate this iter */
-			iter->cursor = NULL;
-			
-			if (header == NULL) {
-				/* invalidating because HeaderList is
-				 * being destroyed */
-				list_unlink ((ListNode *) iter);
-				iter->headers = NULL;
-			}
-		}
-		
-		iter = next;
-	}
-}
-
-
 /**
  * g_mime_header_list_new:
  *
@@ -519,8 +527,8 @@ g_mime_header_list_new (void)
 						  g_free, NULL);
 	headers->hash = g_hash_table_new (g_mime_strcase_hash,
 					  g_mime_strcase_equal);
-	list_init (&headers->iters);
 	list_init (&headers->list);
+	headers->version = 0;
 	headers->raw = NULL;
 	
 	return headers;
@@ -537,14 +545,10 @@ void
 g_mime_header_list_destroy (GMimeHeaderList *headers)
 {
 	GMimeHeader *header, *next;
-	GMimeHeaderIter *iter;
 	
 	if (!headers)
 		return;
 	
-	/* invalidate all our outstanding iterators */
-	g_mime_header_list_invalidate_iters (headers, NULL);
-	
 	header = (GMimeHeader *) headers->list.head;
 	while (header->next) {
 		next = header->next;
@@ -707,7 +711,7 @@ g_mime_header_list_remove (GMimeHeaderList *headers, const char *name)
 	}
 	
 	/* invalidate all our outstanding iterators matching @header */
-	g_mime_header_list_invalidate_iters (headers, header);
+	headers->version++;
 	
 	/* remove/free the header */
 	list_unlink ((ListNode *) header);
@@ -721,26 +725,29 @@ g_mime_header_list_remove (GMimeHeaderList *headers, const char *name)
 /**
  * g_mime_header_list_get_iter:
  * @headers: a #GMimeHeaderList
+ * @iter: a #GMimeHeaderIter
  *
- * Gets a new iterator for traversing @headers.
+ * Initializes an iterator for traversing @headers.
  *
- * Returns: a new #GMimeHeaderIter which must be freed using
- * g_mime_header_iter_free() when finished with it.
+ * Returns: a %TRUE if successful or %FALSE if there are no headers to
+ * traverse.
  **/
-GMimeHeaderIter *
-g_mime_header_list_get_iter (GMimeHeaderList *headers)
+gboolean
+g_mime_header_list_get_iter (GMimeHeaderList *headers, GMimeHeaderIter *iter)
 {
-	GMimeHeaderIter *iter;
+	GMimeHeader *cursor;
 	
-	g_return_val_if_fail (headers != NULL, NULL);
+	g_return_val_if_fail (headers != NULL, FALSE);
 	
-	iter = g_new (GMimeHeaderIter, 1);
-	iter->cursor = (GMimeHeader *) headers->list.head;
-	iter->headers = headers;
+	cursor = (GMimeHeader *) headers->list.head;
+	if (!cursor->next)
+		return FALSE;
 	
-	list_append (&headers->iters, (ListNode *) iter);
+	iter->version = headers->version;
+	iter->hdrlist = headers;
+	iter->cursor = cursor;
 	
-	return iter;
+	return TRUE;
 }
 
 
