@@ -50,11 +50,11 @@ static void g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass
 static void g_mime_message_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static void message_init (GMimeObject *object);
-static void message_add_header (GMimeObject *object, const char *header, const char *value);
+static void message_prepend_header (GMimeObject *object, const char *header, const char *value);
+static void message_append_header (GMimeObject *object, const char *header, const char *value);
 static void message_set_header (GMimeObject *object, const char *header, const char *value);
 static const char *message_get_header (GMimeObject *object, const char *header);
-static void message_remove_header (GMimeObject *object, const char *header);
+static gboolean message_remove_header (GMimeObject *object, const char *header);
 static char *message_get_headers (GMimeObject *object);
 static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream);
 
@@ -80,7 +80,6 @@ static char *rfc822_headers[] = {
 	"Sender",
 	"To",
 	"Cc",
-	NULL
 };
 
 
@@ -119,11 +118,11 @@ g_mime_message_class_init (GMimeMessageClass *klass)
 	
 	gobject_class->finalize = g_mime_message_finalize;
 	
-	object_class->init = message_init;
-	object_class->add_header = message_add_header;
+	object_class->prepend_header = message_prepend_header;
+	object_class->append_header = message_append_header;
+	object_class->remove_header = message_remove_header;
 	object_class->set_header = message_set_header;
 	object_class->get_header = message_get_header;
-	object_class->remove_header = message_remove_header;
 	object_class->get_headers = message_get_headers;
 	object_class->write_to_stream = message_write_to_stream;
 }
@@ -131,31 +130,33 @@ g_mime_message_class_init (GMimeMessageClass *klass)
 static void
 g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 {
+	GMimeHeaderList *headers = ((GMimeObject *) message)->headers;
+	
 	message->from = NULL;
 	message->reply_to = NULL;
 	message->recipients = g_hash_table_new (g_str_hash, g_str_equal);
 	message->subject = NULL;
 	message->date = 0;
-	message->gmt_offset = 0;
+	message->tz_offset = 0;
 	message->message_id = NULL;
 	message->mime_part = NULL;
 	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Sender", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "From", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "To", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Cc", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Bcc", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Sender", write_addrspec);
+	g_mime_header_list_register_writer (headers, "From", write_addrspec);
+	g_mime_header_list_register_writer (headers, "To", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Cc", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Bcc", write_addrspec);
 	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Sender", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-From", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-To", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Cc", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Bcc", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-Sender", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-From", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-To", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-Cc", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-Bcc", write_addrspec);
 	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Subject", write_subject);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Received", write_received);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Message-Id", write_msgid);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "References", write_structured);
+	g_mime_header_list_register_writer (headers, "Subject", write_subject);
+	g_mime_header_list_register_writer (headers, "Received", write_received);
+	g_mime_header_list_register_writer (headers, "Message-Id", write_msgid);
+	g_mime_header_list_register_writer (headers, "References", write_structured);
 }
 
 static gboolean
@@ -192,15 +193,7 @@ g_mime_message_finalize (GObject *object)
 }
 
 
-static void
-message_init (GMimeObject *object)
-{
-	/* no-op */
-	GMIME_OBJECT_CLASS (parent_class)->init (object);
-}
-
-
-typedef void (*token_skip_t) (const char **in);
+typedef void (* token_skip_t) (const char **in);
 
 struct _received_token {
 	char *token;
@@ -609,7 +602,7 @@ enum {
 	HEADER_UNKNOWN
 };
 
-static char *headers[] = {
+static const char *message_headers[] = {
 	"From",
 	"Reply-To",
 	"To",
@@ -618,7 +611,6 @@ static char *headers[] = {
 	"Subject",
 	"Date",
 	"Message-Id",
-	NULL
 };
 
 static gboolean
@@ -629,8 +621,8 @@ process_header (GMimeObject *object, const char *header, const char *value)
 	int offset, i;
 	time_t date;
 	
-	for (i = 0; headers[i]; i++) {
-		if (!g_ascii_strcasecmp (headers[i], header))
+	for (i = 0; i < G_N_ELEMENTS (message_headers); i++) {
+		if (!g_ascii_strcasecmp (message_headers[i], header))
 			break;
 	}
 	
@@ -664,7 +656,7 @@ process_header (GMimeObject *object, const char *header, const char *value)
 		if (value) {
 			date = g_mime_utils_header_decode_date (value, &offset);
 			message->date = date;
-			message->gmt_offset = offset;
+			message->tz_offset = offset;
 		}
 		break;
 	case HEADER_MESSAGE_ID:
@@ -673,70 +665,96 @@ process_header (GMimeObject *object, const char *header, const char *value)
 		break;
 	default:
 		return FALSE;
-		break;
 	}
 	
 	return TRUE;
 }
 
 static void
-message_add_header (GMimeObject *object, const char *header, const char *value)
+message_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
+	GMimeMessage *message = (GMimeMessage *) object;
+	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
+		return;
 	
-	if (g_ascii_strncasecmp ("Content-", header, 8)) {
-		if (process_header (object, header, value))
-			g_mime_header_add (object->headers, header, value);
-		else
-			GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
-	}
+	if (!process_header (object, header, value))
+		GMIME_OBJECT_CLASS (parent_class)->prepend_header (object, header, value);
+	else
+		g_mime_header_list_prepend (object->headers, header, value);
 	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
+	if (message->mime_part)
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
+}
+
+static void
+message_append_header (GMimeObject *object, const char *header, const char *value)
+{
+	GMimeMessage *message = (GMimeMessage *) object;
+	
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
+		return;
+	
+	/* Make sure that the header is not a Content-* header, else it
+           doesn't belong on a message */
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
+		return;
+	
+	if (!process_header (object, header, value))
+		GMIME_OBJECT_CLASS (parent_class)->append_header (object, header, value);
+	else
+		g_mime_header_list_append (object->headers, header, value);
+	
+	if (message->mime_part)
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
 }
 
 static void
 message_set_header (GMimeObject *object, const char *header, const char *value)
 {
+	GMimeMessage *message = (GMimeMessage *) object;
+	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
+		return;
 	
-	if (g_ascii_strncasecmp ("Content-", header, 8)) {
-		if (process_header (object, header, value))
-			g_mime_header_set (object->headers, header, value);
-		else
-			GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
-	}
+	if (!process_header (object, header, value))
+		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
+	else
+		g_mime_header_list_set (object->headers, header, value);
 	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
+	if (message->mime_part)
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
 }
 
 static const char *
 message_get_header (GMimeObject *object, const char *header)
 {
-	/* Make sure that the header is not a Content-* header, else it
-           doesn't belong on a message */
+	GMimeMessage *message = (GMimeMessage *) object;
 	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return "1.0";
 	
+	/* Make sure that the header is not a Content-* header, else it
+           doesn't belong on a message */
 	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
-	else if (((GMimeMessage *) object)->mime_part)
-		return g_mime_object_get_header (((GMimeMessage *) object)->mime_part, header);
+	else if (message->mime_part)
+		return g_mime_object_get_header (message->mime_part, header);
 	else
 		return NULL;
 }
 
-static void
+static gboolean
 message_remove_header (GMimeObject *object, const char *header)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
@@ -744,16 +762,19 @@ message_remove_header (GMimeObject *object, const char *header)
 	int i;
 	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return;
+		return FALSE;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
+	if (!g_ascii_strncasecmp ("Content-", header, 8)) {
+		if (message->mime_part)
+			return g_mime_object_remove_header (message->mime_part, header);
+		
+		return FALSE;
+	}
 	
-	if (!g_ascii_strncasecmp ("Content-", header, 8))
-		return;
-	
-	for (i = 0; headers[i]; i++) {
-		if (!g_ascii_strcasecmp (headers[i], header))
+	for (i = 0; i < G_N_ELEMENTS (message_headers); i++) {
+		if (!g_ascii_strcasecmp (message_headers[i], header))
 			break;
 	}
 	
@@ -787,7 +808,7 @@ message_remove_header (GMimeObject *object, const char *header)
 		break;
 	case HEADER_DATE:
 		message->date = 0;
-		message->gmt_offset = 0;
+		message->tz_offset = 0;
 		break;
 	case HEADER_MESSAGE_ID:
 		g_free (message->message_id);
@@ -797,10 +818,10 @@ message_remove_header (GMimeObject *object, const char *header)
 		break;
 	}
 	
-	GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
+	if (message->mime_part)
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
 	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
+	return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
 
@@ -816,14 +837,14 @@ message_get_headers (GMimeObject *object)
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), ba);
 	
-	if (message->mime_part && g_mime_header_has_raw (message->mime_part->headers)) {
+	if (message->mime_part && g_mime_header_list_has_raw (message->mime_part->headers)) {
 		/* if the mime part has raw headers, then it contains the message headers as well */
-		g_mime_header_write_to_stream (message->mime_part->headers, stream);
+		g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 	} else {
-		g_mime_header_write_to_stream (object->headers, stream);
+		g_mime_header_list_write_to_stream (object->headers, stream);
 		if (message->mime_part) {
 			g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
-			g_mime_header_write_to_stream (message->mime_part->headers, stream);
+			g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 		}
 	}
 	
@@ -842,10 +863,10 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	ssize_t nwritten, total = 0;
 	
 	if (message->mime_part) {
-		if (!g_mime_header_has_raw (message->mime_part->headers)) {
+		if (!g_mime_header_list_has_raw (message->mime_part->headers)) {
 			/* if the mime part has raw headers, then it contains the message
 			 * headers as well otherwise it doesn't, so write them now */
-			if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+			if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
 				return -1;
 			
 			total += nwritten;
@@ -859,7 +880,7 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		if ((nwritten = g_mime_object_write_to_stream (message->mime_part, stream)) == -1)
 			return -1;
 	} else {
-		if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+		if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -883,11 +904,12 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * strictly a cosmetic thing, so if you are unsure, it is safe to say
  * no (%FALSE).
  *
- * Returns an empty MIME Message object.
+ * Returns: an empty MIME Message object.
  **/
 GMimeMessage *
 g_mime_message_new (gboolean pretty_headers)
 {
+	GMimeHeaderList *headers;
 	GMimeMessage *message;
 	int i;
 	
@@ -895,8 +917,9 @@ g_mime_message_new (gboolean pretty_headers)
 	
 	if (pretty_headers) {
 		/* Populate with the "standard" rfc822 headers so we can have a standard order */
-		for (i = 0; rfc822_headers[i]; i++) 
-			g_mime_header_set (GMIME_OBJECT (message)->headers, rfc822_headers[i], NULL);
+		headers = ((GMimeObject *) message)->headers;
+		for (i = 0; i < G_N_ELEMENTS (rfc822_headers); i++) 
+			g_mime_header_list_set (headers, rfc822_headers[i], NULL);
 	}
 	
 	return message;
@@ -927,7 +950,7 @@ g_mime_message_set_sender (GMimeMessage *message, const char *sender)
 	encoded = internet_address_list_to_string (addrlist, TRUE);
 	internet_address_list_destroy (addrlist);
 	
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "From", encoded);
+	g_mime_header_list_set (GMIME_OBJECT (message)->headers, "From", encoded);
 	g_free (encoded);
 }
 
@@ -938,7 +961,7 @@ g_mime_message_set_sender (GMimeMessage *message, const char *sender)
  *
  * Gets the email address of the sender from @message.
  *
- * Returns the sender's name and address of the MIME Message.
+ * Returns: the sender's name and address of the MIME Message.
  **/
 const char *
 g_mime_message_get_sender (GMimeMessage *message)
@@ -965,7 +988,7 @@ g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
 	g_free (message->reply_to);
 	message->reply_to = g_strstrip (g_strdup (reply_to));
 	
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "Reply-To", message->reply_to);
+	g_mime_header_list_set (GMIME_OBJECT (message)->headers, "Reply-To", message->reply_to);
 }
 
 
@@ -975,7 +998,7 @@ g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
  *
  * Gets the Reply-To address from @message.
  *
- * Returns the sender's Reply-To address from the MIME Message.
+ * Returns: the sender's Reply-To address from the MIME Message.
  **/
 const char *
 g_mime_message_get_reply_to (GMimeMessage *message)
@@ -995,10 +1018,10 @@ sync_recipient_header (GMimeMessage *message, const char *type)
 	/* sync the specified recipient header */
 	if ((recipients = g_mime_message_get_recipients (message, type))) {
 		string = internet_address_list_to_string (recipients, TRUE);
-		g_mime_header_set (GMIME_OBJECT (message)->headers, type, string);
+		g_mime_header_list_set (GMIME_OBJECT (message)->headers, type, string);
 		g_free (string);
 	} else
-		g_mime_header_set (GMIME_OBJECT (message)->headers, type, NULL);
+		g_mime_header_list_set (GMIME_OBJECT (message)->headers, type, NULL);
 }
 
 
@@ -1088,7 +1111,7 @@ g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, co
  * recipient types include: #GMIME_RECIPIENT_TYPE_TO,
  * #GMIME_RECIPIENT_TYPE_CC and #GMIME_RECIPIENT_TYPE_BCC.
  *
- * Returns a list of recipients of a chosen type from the MIME
+ * Returns: a list of recipients of a chosen type from the MIME
  * Message.
  **/
 const InternetAddressList *
@@ -1113,7 +1136,7 @@ static const char *recipient_types[] = {
  *
  * Gets the complete list of recipients for @message.
  *
- * Returns a newly allocated #InternetAddressList containing all
+ * Returns: a newly allocated #InternetAddressList containing all
  * recipients of the message.
  **/
 InternetAddressList *
@@ -1164,7 +1187,7 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
 	message->subject = g_strstrip (g_strdup (subject));
 	
 	encoded = g_mime_utils_header_encode_text (message->subject);
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "Subject", encoded);
+	g_mime_object_set_header (GMIME_OBJECT (message), "Subject", encoded);
 	g_free (encoded);
 }
 
@@ -1175,7 +1198,7 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
  *
  * Gets the message's subject.
  *
- * Returns the unencoded UTF-8 Subject field on a MIME Message.
+ * Returns: the unencoded UTF-8 Subject field on a MIME Message.
  **/
 const char *
 g_mime_message_get_subject (GMimeMessage *message)
@@ -1189,68 +1212,90 @@ g_mime_message_get_subject (GMimeMessage *message)
 /**
  * g_mime_message_set_date:
  * @message: MIME Message
- * @date: Sent-date (ex: gotten from time (NULL))
- * @gmt_offset: GMT date offset (in +/- hours)
+ * @date: a date to be used in the Date header
+ * @tz_offset: timezone offset (in +/- hours)
  * 
- * Sets the sent-date on a MIME Message.
+ * Sets the Date header on a MIME Message.
  **/
 void
-g_mime_message_set_date (GMimeMessage *message, time_t date, int gmt_offset)
+g_mime_message_set_date (GMimeMessage *message, time_t date, int tz_offset)
 {
-	char *date_str;
+	char *str;
 	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	message->date = date;
-	message->gmt_offset = gmt_offset;
+	message->tz_offset = tz_offset;
 	
-	date_str = g_mime_message_get_date_string (message);
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "Date", date_str);
-	g_free (date_str);
+	str = g_mime_utils_header_format_date (date, tz_offset);
+	g_mime_object_set_header (GMIME_OBJECT (message), "Date", str);
+	g_free (str);
 }
 
 
 /**
  * g_mime_message_get_date:
  * @message: MIME Message
- * @date: Sent-date
- * @gmt_offset: GMT date offset (in +/- hours)
+ * @date: pointer to a date in time_t
+ * @tz_offset: pointer to timezone offset (in +/- hours)
  * 
- * Stores the date in time_t format in #date and the GMT offset in
- * @gmt_offset.
+ * Stores the date in time_t format in @date. If @tz_offset is
+ * non-%NULL, then the timezone offset in will be stored in
+ * @tz_offset.
  **/
 void
-g_mime_message_get_date (GMimeMessage *message, time_t *date, int *gmt_offset)
+g_mime_message_get_date (GMimeMessage *message, time_t *date, int *tz_offset)
 {
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (date != NULL);
 	
 	*date = message->date;
 	
-	if (gmt_offset)
-		*gmt_offset = message->gmt_offset;
+	if (tz_offset)
+		*tz_offset = message->tz_offset;
 }
 
 
 /**
- * g_mime_message_get_date_string:
+ * g_mime_message_get_date_as_string:
  * @message: MIME Message
  *
- * Gets the message's sent date in string format.
+ * Gets the message's sent-date in string format.
  * 
- * Returns the sent-date of the MIME Message in string format.
+ * Returns: a newly allocated string containing the Date header value.
  **/
 char *
-g_mime_message_get_date_string (GMimeMessage *message)
+g_mime_message_get_date_as_string (GMimeMessage *message)
 {
-	char *date_str;
-	
 	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	date_str = g_mime_utils_header_format_date (message->date,
-						    message->gmt_offset);
+	return g_mime_utils_header_format_date (message->date, message->tz_offset);
+}
+
+
+/**
+ * g_mime_message_set_date_as_string:
+ * @message: MIME Message
+ * @str: a date string
+ *
+ * Sets the sent-date of the message.
+ **/
+void
+g_mime_message_set_date_as_string (GMimeMessage *message, const char *str)
+{
+	int tz_offset;
+	time_t date;
+	char *buf;
 	
-	return date_str;
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	
+	date = g_mime_utils_header_decode_date (str, &tz_offset);
+	message->tz_offset = tz_offset;
+	message->date = date;
+	
+	buf = g_mime_utils_header_format_date (date, tz_offset);
+	g_mime_object_set_header (GMIME_OBJECT (message), "Date", buf);
+	g_free (buf);
 }
 
 
@@ -1273,7 +1318,7 @@ g_mime_message_set_message_id (GMimeMessage *message, const char *message_id)
 	message->message_id = g_strstrip (g_strdup (message_id));
 	
 	msgid = g_strdup_printf ("<%s>", message_id);
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "Message-Id", msgid);
+	g_mime_object_set_header (GMIME_OBJECT (message), "Message-Id", msgid);
 	g_free (msgid);
 }
 
@@ -1284,7 +1329,7 @@ g_mime_message_set_message_id (GMimeMessage *message, const char *message_id)
  *
  * Gets the Message-Id header of @message.
  *
- * Returns the Message-Id of a message.
+ * Returns: the Message-Id of a message.
  **/
 const char *
 g_mime_message_get_message_id (GMimeMessage *message)
@@ -1295,83 +1340,13 @@ g_mime_message_get_message_id (GMimeMessage *message)
 }
 
 
-/**
- * g_mime_message_add_header: Add an arbitrary message header
- * @message: MIME Message
- * @header: rfc822 header field
- * @value: the contents of the header field
- *
- * Add an arbitrary message header to the MIME Message such as
- * X-Mailer, X-Priority, or In-Reply-To.
- *
- * WARNING: This interface is deprecated. Use
- * g_mime_object_add_header() instead.
- **/
-void
-g_mime_message_add_header (GMimeMessage *message, const char *header, const char *value)
-{
-	g_return_if_fail (GMIME_IS_MESSAGE (message));
-	g_return_if_fail (header != NULL);
-	g_return_if_fail (value != NULL);
-	
-	g_mime_object_add_header (GMIME_OBJECT (message), header, value);
-}
-
-
-/**
- * g_mime_message_set_header: Add an arbitrary message header
- * @message: MIME Message
- * @header: rfc822 header field
- * @value: the contents of the header field
- *
- * Set an arbitrary message header to the MIME Message such as
- * X-Mailer, X-Priority, or In-Reply-To.
- *
- * WARNING: This interface is deprecated. Use
- * g_mime_object_set_header().
- **/
-void
-g_mime_message_set_header (GMimeMessage *message, const char *header, const char *value)
-{
-	g_return_if_fail (GMIME_IS_MESSAGE (message));
-	g_return_if_fail (header != NULL);
-	g_return_if_fail (value != NULL);
-	
-	g_mime_object_set_header (GMIME_OBJECT (message), header, value);
-}
-
-
-/**
- * g_mime_message_get_header:
- * @message: MIME Message
- * @header: rfc822 header field
- *
- * Gets the value of the requested header @header if it exists, or
- * %NULL otherwise.
- *
- * WARNING: This interface is deprecated. Use
- * g_mime_message_get_header().
- *
- * Returns the value of the requested header (or %NULL if it isn't
- * set)
- **/
-const char *
-g_mime_message_get_header (GMimeMessage *message, const char *header)
-{
-	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
-	g_return_val_if_fail (header != NULL, NULL);
-	
-	return g_mime_object_get_header (GMIME_OBJECT (message), header);
-}
-
-
 /**
  * g_mime_message_get_mime_part:
  * @message: MIME Message
  *
  * Gets the toplevel MIME part contained within @message.
  *
- * Returns the toplevel MIME part of @message.
+ * Returns: the toplevel MIME part of @message.
  **/
 GMimeObject *
 g_mime_message_get_mime_part (GMimeMessage *message)
@@ -1398,12 +1373,13 @@ void
 g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 {
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (GMIME_IS_OBJECT (mime_part));
 	
 	g_object_ref (mime_part);
-	g_mime_header_set_raw (mime_part->headers, NULL);
+	g_mime_header_list_set_raw (mime_part->headers, NULL);
 	
 	if (message->mime_part) {
-		g_mime_header_set_raw (message->mime_part->headers, NULL);
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
 		g_object_unref (message->mime_part);
 	}
 	
@@ -1411,110 +1387,6 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 }
 
 
-/* The proper way to handle a multipart/alternative part is to return
- * the last part that we know how to render. For our purposes, we are
- * going to assume:
- *
- * If @want_plain is %FALSE then assume we can render text/plain and
- * text/html, thus the order of preference is text/html, then
- * text/plain and finally text/<any>.
- *
- * Otherwise, if @want_plain is %TRUE then we assume that we do not
- * know how to render text/html and so our order of preference becomes
- * text/plain and then text/<any>.
- **/
-static GMimeObject *
-handle_multipart_alternative (GMimeMultipart *multipart, gboolean want_plain, gboolean *is_html)
-{
-	GMimeObject *mime_part, *text_part = NULL;
-	const GMimeContentType *type;
-	GList *subpart;
-	
-	subpart = multipart->subparts;
-	while (subpart) {
-		mime_part = subpart->data;
-		
-		type = g_mime_object_get_content_type (mime_part);
-		if (g_mime_content_type_is_type (type, "text", "*")) {
-			if (!text_part || !g_ascii_strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
-				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
-				text_part = mime_part;
-			}
-		}
-		
-		subpart = subpart->next;
-	}
-	
-	return text_part;
-}
-
-static GMimeObject *
-handle_multipart_mixed (GMimeMultipart *multipart, gboolean want_plain, gboolean *is_html)
-{
-	GMimeObject *mime_part, *text_part = NULL;
-	const GMimeContentType *type, *first_type = NULL;
-	GList *subpart;
-	
-	subpart = multipart->subparts;
-	while (subpart) {
-		mime_part = subpart->data;
-		
-		type = g_mime_object_get_content_type (mime_part);
-		if (GMIME_IS_MULTIPART (mime_part)) {
-			multipart = GMIME_MULTIPART (mime_part);
-			if (g_mime_content_type_is_type (type, "multipart", "alternative")) {
-				mime_part = handle_multipart_alternative (multipart, want_plain, is_html);
-				if (mime_part)
-					return mime_part;
-			} else {
-				mime_part = handle_multipart_mixed (multipart, want_plain, is_html);
-				if (mime_part && !text_part)
-					text_part = mime_part;
-			}
-		} else if (g_mime_content_type_is_type (type, "text", "*")) {
-			if (!g_ascii_strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
-				/* we got what we came for */
-				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
-				return mime_part;
-			}
-			
-			/* if we haven't yet found a text part or if it is a type we can
-			 * understand and it is the first of that type, save it */
-			if (!text_part || (!g_ascii_strcasecmp (type->subtype, "plain") && (first_type &&
-					   g_ascii_strcasecmp (type->subtype, first_type->subtype) != 0))) {
-				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
-				text_part = mime_part;
-				first_type = type;
-			}
-		}
-		
-		subpart = subpart->next;
-	}
-	
-	return text_part;
-}
-
-
-/**
- * g_mime_message_get_headers:
- * @message: MIME Message
- *
- * Allocates a string buffer containing the raw message headers.
- *
- * WARNING: This interface is deprecated. Use
- * g_mime_object_get_headers() instead.
- *
- * Returns an allocated string containing the raw message headers.
- **/
-char *
-g_mime_message_get_headers (GMimeMessage *message)
-{
-	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
-	
-	return g_mime_object_get_headers (GMIME_OBJECT (message));
-}
-
-
 /**
  * g_mime_message_foreach_part:
  * @message: MIME message
