@@ -58,11 +58,11 @@ static void g_mime_object_class_init (GMimeObjectClass *klass);
 static void g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass);
 static void g_mime_object_finalize (GObject *object);
 
-static void init (GMimeObject *object);
-static void add_header (GMimeObject *object, const char *header, const char *value);
-static void set_header (GMimeObject *object, const char *header, const char *value);
-static const char *get_header (GMimeObject *object, const char *header);
-static void remove_header (GMimeObject *object, const char *header);
+static void prepend_header (GMimeObject *object, const char *name, const char *value);
+static void append_header (GMimeObject *object, const char *name, const char *value);
+static void set_header (GMimeObject *object, const char *name, const char *value);
+static const char *get_header (GMimeObject *object, const char *name);
+static gboolean remove_header (GMimeObject *object, const char *name);
 static void set_content_type (GMimeObject *object, GMimeContentType *content_type);
 static char *get_headers (GMimeObject *object);
 static ssize_t write_to_stream (GMimeObject *object, GMimeStream *stream);
@@ -112,11 +112,11 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 	
 	object_class->finalize = g_mime_object_finalize;
 	
-	klass->init = init;
-	klass->add_header = add_header;
+	klass->prepend_header = prepend_header;
+	klass->append_header = append_header;
+	klass->remove_header = remove_header;
 	klass->set_header = set_header;
 	klass->get_header = get_header;
-	klass->remove_header = remove_header;
 	klass->set_content_type = set_content_type;
 	klass->get_headers = get_headers;
 	klass->write_to_stream = write_to_stream;
@@ -127,13 +127,13 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 static void
 g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass)
 {
-	object->headers = g_mime_header_new ();
+	object->headers = g_mime_header_list_new ();
 	object->content_type = NULL;
 	object->disposition = NULL;
 	object->content_id = NULL;
 	
-	g_mime_header_register_writer (object->headers, "Content-Type", write_content_type);
-	g_mime_header_register_writer (object->headers, "Content-Disposition", write_disposition);
+	g_mime_header_list_register_writer (object->headers, "Content-Type", write_content_type);
+	g_mime_header_list_register_writer (object->headers, "Content-Disposition", write_disposition);
 }
 
 static void
@@ -148,7 +148,7 @@ g_mime_object_finalize (GObject *object)
 		g_mime_content_disposition_destroy (mime->disposition);
 	
 	if (mime->headers)
-		g_mime_header_destroy (mime->headers);
+		g_mime_header_list_destroy (mime->headers);
 	
 	g_free (mime->content_id);
 	
@@ -205,7 +205,7 @@ _g_mime_object_content_type_changed (GMimeObject *object)
 	g_string_free (string, FALSE);
 	
 	type = p + strlen ("Content-Type: ");
-	g_mime_header_set (object->headers, "Content-Type", type);
+	g_mime_header_list_set (object->headers, "Content-Type", type);
 	g_free (p);
 }
 
@@ -238,10 +238,10 @@ _g_mime_object_content_disposition_changed (GMimeObject *object)
 	
 	if (object->disposition) {
 		str = g_mime_content_disposition_to_string (object->disposition, FALSE);
-		g_mime_header_set (object->headers, "Content-Disposition", str);
+		g_mime_header_list_set (object->headers, "Content-Disposition", str);
 		g_free (str);
 	} else {
-		g_mime_header_remove (object->headers, "Content-Disposition");
+		g_mime_header_list_remove (object->headers, "Content-Disposition");
 	}
 }
 
@@ -285,13 +285,6 @@ g_mime_object_register_type (const char *type, const char *subtype, GType object
 }
 
 
-static void
-init (GMimeObject *object)
-{
-	/* no-op */
-}
-
-
 /**
  * g_mime_object_new_type:
  * @type: mime type
@@ -304,7 +297,7 @@ init (GMimeObject *object)
  * class that can handle @type. If that also fails, then it will use
  * the generic part class, #GMimePart.
  *
- * Returns an appropriate #GMimeObject registered to handle mime-types
+ * Returns: an appropriate #GMimeObject registered to handle mime-types
  * of @type/@subtype.
  **/
 GMimeObject *
@@ -342,8 +335,6 @@ g_mime_object_new_type (const char *type, const char *subtype)
 	
 	object = g_object_new (obj_type, NULL);
 	
-	GMIME_OBJECT_GET_CLASS (object)->init (object);
-	
 	return object;
 }
 
@@ -362,7 +353,7 @@ set_content_type (GMimeObject *object, GMimeContentType *content_type)
 
 /**
  * g_mime_object_set_content_type:
- * @object: Mime object
+ * @object: a #GMimeObject
  * @mime_type: Mime type
  *
  * Sets the content-type for the specified Mime object.
@@ -379,12 +370,12 @@ g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *mime_type
 
 /**
  * g_mime_object_get_content_type:
- * @object: Mime object
+ * @object: a #GMimeObject
  *
- * Gets the Content-Type object for the given Mime object or %NULL on
- * fail.
+ * Gets the #GMimeContentType object for the given Mime object or
+ * %NULL on fail.
  *
- * Returns the content-type object for the specified Mime object.
+ * Returns: the content-type object for the specified Mime object.
  **/
 const GMimeContentType *
 g_mime_object_get_content_type (GMimeObject *object)
@@ -397,7 +388,7 @@ g_mime_object_get_content_type (GMimeObject *object)
 
 /**
  * g_mime_object_set_content_type_parameter:
- * @object: Mime object
+ * @object: a #GMimeObject
  * @name: param name
  * @value: param value
  *
@@ -415,13 +406,13 @@ g_mime_object_set_content_type_parameter (GMimeObject *object, const char *name,
 
 /**
  * g_mime_object_get_content_type_parameter:
- * @object: Mime object
+ * @object: a #GMimeObject
  * @name: param name
  *
  * Gets the value of the content-type param @name set on the Mime part
  * @object.
  *
- * Returns the value of the requested content-type param or %NULL on
+ * Returns: the value of the requested content-type param or %NULL on
  * if the param doesn't exist.
  **/
 const char *
@@ -436,11 +427,11 @@ g_mime_object_get_content_type_parameter (GMimeObject *object, const char *name)
 
 /**
  * g_mime_object_get_content_disposition:
- * @object: Mime object
+ * @object: a #GMimeObject
  *
- * Get the content disposition for the specified Mime object.
+ * Gets the #GMimeContentDisposition for the specified Mime object.
  *
- * Returns the #GMimeContentDisposition set on the Mime object.
+ * Returns: the #GMimeContentDisposition set on the Mime object.
  **/
 const GMimeContentDisposition *
 g_mime_object_get_content_disposition (GMimeObject *object)
@@ -453,7 +444,7 @@ g_mime_object_get_content_disposition (GMimeObject *object)
 
 /**
  * g_mime_object_set_content_disposition:
- * @object: Mime object
+ * @object: a #GMimeObject
  * @disposition: a #GMimeContentDisposition object
  *
  * Set the content disposition for the specified mime part.
@@ -475,7 +466,7 @@ g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposit
 
 /**
  * g_mime_object_set_disposition:
- * @object: Mime object
+ * @object: a #GMimeObject
  * @disposition: disposition ("attachment" or "inline")
  *
  * Sets the disposition to @disposition which may be one of
@@ -500,11 +491,11 @@ g_mime_object_set_disposition (GMimeObject *object, const char *disposition)
 
 /**
  * g_mime_object_get_disposition:
- * @object: Mime object
+ * @object: a #GMimeObject
  *
  * Gets the Mime object's disposition if set or %NULL otherwise.
  *
- * Returns the disposition string which is probably one of
+ * Returns: the disposition string which is probably one of
  * #GMIME_DISPOSITION_ATTACHMENT or #GMIME_DISPOSITION_INLINE.
  **/
 const char *
@@ -521,7 +512,7 @@ g_mime_object_get_disposition (GMimeObject *object)
 
 /**
  * g_mime_object_set_content_disposition_parameter:
- * @object: Mime object
+ * @object: a #GMimeObject
  * @attribute: parameter name
  * @value: parameter value
  *
@@ -544,13 +535,13 @@ g_mime_object_set_content_disposition_parameter (GMimeObject *object, const char
 
 /**
  * g_mime_object_get_content_disposition_parameter:
- * @object: Mime object
+ * @object: a #GMimeObject
  * @attribute: parameter name
  *
  * Gets the value of the Content-Disposition parameter specified by
  * @attribute, or %NULL if the parameter does not exist.
  *
- * Returns the value of a previously defined content-disposition
+ * Returns: the value of a previously defined content-disposition
  * parameter specified by @attribute.
  **/
 const char *
@@ -568,7 +559,7 @@ g_mime_object_get_content_disposition_parameter (GMimeObject *object, const char
 
 /**
  * g_mime_object_set_content_id:
- * @object: Mime object
+ * @object: a #GMimeObject
  * @content_id: content-id (addr-spec portion)
  *
  * Sets the Content-Id of the Mime object.
@@ -591,11 +582,11 @@ g_mime_object_set_content_id (GMimeObject *object, const char *content_id)
 
 /**
  * g_mime_object_get_content_id:
- * @object: Mime object
+ * @object: a #GMimeObject
  *
  * Gets the Content-Id of the Mime object or NULL if one is not set.
  *
- * Returns a const pointer to the Content-Id header.
+ * Returns: a const pointer to the Content-Id header.
  **/
 const char *
 g_mime_object_get_content_id (GMimeObject *object)
@@ -613,11 +604,10 @@ enum {
 	HEADER_UNKNOWN,
 };
 
-static char *headers[] = {
+static char *content_headers[] = {
 	"Content-Disposition",
 	"Content-Type",
 	"Content-Id",
-	NULL
 };
 
 static gboolean
@@ -627,8 +617,8 @@ process_header (GMimeObject *object, const char *header, const char *value)
 	GMimeContentType *content_type;
 	int i;
 	
-	for (i = 0; headers[i]; i++) {
-		if (!g_ascii_strcasecmp (headers[i], header))
+	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
+		if (!g_ascii_strcasecmp (content_headers[i], header))
 			break;
 	}
 	
@@ -651,35 +641,61 @@ process_header (GMimeObject *object, const char *header, const char *value)
 		return FALSE;
 	}
 	
-	g_mime_header_set (object->headers, header, value);
+	g_mime_header_list_set (object->headers, header, value);
 	
 	return TRUE;
 }
 
 static void
-add_header (GMimeObject *object, const char *header, const char *value)
+prepend_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
-		g_mime_header_add (object->headers, header, value);
+		g_mime_header_list_prepend (object->headers, header, value);
 }
 
 
 /**
- * g_mime_object_add_header:
- * @object: mime object
+ * g_mime_object_prepend_header:
+ * @object: a #GMimeObject
  * @header: header name
  * @value: header value
  *
- * Adds an arbitrary header to the Mime object.
+ * Prepends a header to the Mime object.
  **/
 void
-g_mime_object_add_header (GMimeObject *object, const char *header, const char *value)
+g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	g_return_if_fail (header != NULL);
 	g_return_if_fail (value != NULL);
 	
-	GMIME_OBJECT_GET_CLASS (object)->add_header (object, header, value);
+	GMIME_OBJECT_GET_CLASS (object)->prepend_header (object, header, value);
+}
+
+static void
+append_header (GMimeObject *object, const char *header, const char *value)
+{
+	if (!process_header (object, header, value))
+		g_mime_header_list_append (object->headers, header, value);
+}
+
+
+/**
+ * g_mime_object_append_header:
+ * @object: a #GMimeObject
+ * @header: header name
+ * @value: header value
+ *
+ * Appends a header to the Mime object.
+ **/
+void
+g_mime_object_append_header (GMimeObject *object, const char *header, const char *value)
+{
+	g_return_if_fail (GMIME_IS_OBJECT (object));
+	g_return_if_fail (header != NULL);
+	g_return_if_fail (value != NULL);
+	
+	GMIME_OBJECT_GET_CLASS (object)->append_header (object, header, value);
 }
 
 
@@ -687,13 +703,13 @@ static void
 set_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
-		g_mime_header_set (object->headers, header, value);
+		g_mime_header_list_set (object->headers, header, value);
 }
 
 
 /**
  * g_mime_object_set_header:
- * @object: mime object
+ * @object: a #GMimeObject
  * @header: header name
  * @value: header value
  *
@@ -713,19 +729,19 @@ g_mime_object_set_header (GMimeObject *object, const char *header, const char *v
 static const char *
 get_header (GMimeObject *object, const char *header)
 {
-	return g_mime_header_get (object->headers, header);
+	return g_mime_header_list_get (object->headers, header);
 }
 
 
 /**
  * g_mime_object_get_header:
- * @object: mime object
+ * @object: a #GMimeObject
  * @header: header name
  *
  * Gets the value of the requested header if it exists or %NULL
  * otherwise.
  *
- * Returns the value of the header @header if it exists or %NULL
+ * Returns: the value of the header @header if it exists or %NULL
  * otherwise.
  **/
 const char *
@@ -738,45 +754,73 @@ g_mime_object_get_header (GMimeObject *object, const char *header)
 }
 
 
-static void
+static gboolean
 remove_header (GMimeObject *object, const char *header)
 {
-	g_mime_header_remove (object->headers, header);
+	int i;
+	
+	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
+		if (!g_ascii_strcasecmp (content_headers[i], header))
+			break;
+	}
+	
+	switch (i) {
+	case HEADER_CONTENT_DISPOSITION:
+		if (object->disposition) {
+			g_mime_content_disposition_destroy (object->disposition);
+			object->disposition = NULL;
+		}
+		break;
+	case HEADER_CONTENT_TYPE:
+		/* never allow the removal of the Content-Type header */
+		return FALSE;
+	case HEADER_CONTENT_ID:
+		g_free (object->content_id);
+		object->content_id = NULL;
+		break;
+	default:
+		break;
+	}
+	
+	return g_mime_header_list_remove (object->headers, header);
 }
 
 
 /**
  * g_mime_object_remove_header:
- * @object: mime object
+ * @object: a #GMimeObject
  * @header: header name
  *
  * Removed the specified header if it exists.
+ *
+ * Returns: %TRUE if the header was removed or %FALSE if it could not
+ * be found.
  **/
-void
+gboolean
 g_mime_object_remove_header (GMimeObject *object, const char *header)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	g_return_if_fail (header != NULL);
 	
-	GMIME_OBJECT_GET_CLASS (object)->remove_header (object, header);
+	return GMIME_OBJECT_GET_CLASS (object)->remove_header (object, header);
 }
 
 
 static char *
 get_headers (GMimeObject *object)
 {
-	return g_mime_header_to_string (object->headers);
+	return g_mime_header_list_to_string (object->headers);
 }
 
 
 /**
  * g_mime_object_get_headers:
- * @object: mime object
+ * @object: a #GMimeObject
  *
  * Allocates a string buffer containing all of the Mime object's raw
  * headers.
  *
- * Returns an allocated string containing all of the raw Mime headers.
+ * Returns: an allocated string containing all of the raw Mime headers.
  **/
 char *
 g_mime_object_get_headers (GMimeObject *object)
@@ -796,12 +840,12 @@ write_to_stream (GMimeObject *object, GMimeStream *stream)
 
 /**
  * g_mime_object_write_to_stream:
- * @object: mime object
+ * @object: a #GMimeObject
  * @stream: stream
  *
  * Write the contents of the Mime object to @stream.
  *
- * Returns -1 on fail.
+ * Returns: the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream)
@@ -815,11 +859,11 @@ g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream)
 
 /**
  * g_mime_object_to_string:
- * @object: mime object
+ * @object: a #GMimeObject
  *
  * Allocates a string buffer containing the contents of @object.
  *
- * Returns an allocated string containing the contents of the mime
+ * Returns: an allocated string containing the contents of the mime
  * object.
  **/
 char *
@@ -846,6 +890,24 @@ g_mime_object_to_string (GMimeObject *object)
 }
 
 
+/**
+ * g_mime_object_get_header_list:
+ * @object: a #GMimeObject
+ *
+ * Get the header list for @object.
+ *
+ * Returns: the #GMimeHeaderList for @object. Do not free this pointer
+ * when you are done with it.
+ **/
+GMimeHeaderList *
+g_mime_object_get_header_list (GMimeObject *object)
+{
+	g_return_val_if_fail (GMIME_IS_OBJECT (object), NULL);
+	
+	return object->headers;
+}
+
+
 static void
 subtype_bucket_foreach (gpointer key, gpointer value, gpointer user_data)
 {
